#include "core.h"

Core::Core() :
	_hwnd(nullptr),
	_hinst(nullptr),
	_isRunning(true),
	_DXManager(nullptr)
{}

bool Core::Initialize()
{
	kCoreHandle = this;
	if (!CreateMainWindow()) {
		return false;
	}

	_DXManager = new DXManager();
	if (!_DXManager->Initialize(_hwnd, kWidth, kHeight)) {
		return false;
	}

	ShowWindow(_hwnd, SW_SHOW);
	return true;
}

int Core::Run()
{
	MSG msg;
	while (_isRunning) {
		if (PeekMessage(&msg, nullptr, 0, 0, PM_REMOVE)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			if (msg.message == WM_QUIT) {
				_isRunning = false;
			}
		} else {
			OnFrameRender();
		}
	}
	return (int)msg.wParam;
}

void Core::Shutdown()
{
	DestroyMainWindow();
	kCoreHandle = nullptr;
	SAFE_SHUTDOWN(_DXManager);
}

void Core::OnFrameRender()
{
	_DXManager->StartFrameRender();
	//TODO: render something
	_DXManager->EndFrameRender();
}

bool Core::CreateMainWindow()
{
	_hinst = GetModuleHandle(NULL);

	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = WndProc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = _hinst;
	wcex.hIcon = nullptr;
	wcex.hCursor = LoadCursor(nullptr, IDC_ARROW);
	wcex.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wcex.lpszMenuName = nullptr;
	wcex.lpszClassName = kWindowClass;
	wcex.hIconSm = nullptr;

	if (!RegisterClassEx(&wcex)) {
		return false;
	}

	_hwnd = CreateWindow(kWindowClass, kTitle,
		WS_OVERLAPPEDWINDOW,
		0, 0, kWidth, kHeight,
		nullptr, nullptr, _hinst, nullptr);

	if (!_hwnd) {
		return false;
	}
	return true;
}

void Core::DestroyMainWindow()
{
	DestroyWindow(_hwnd);
	UnregisterClass(kTitle, _hinst);
}

LRESULT CALLBACK Core::MessageHandler(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM lparam)
{
	return DefWindowProc(hwnd, umsg, wparam, lparam);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	if ((msg == WM_KEYDOWN) && (wParam == VK_ESCAPE)) {
		PostQuitMessage(EXIT_SUCCESS);
	}

	switch (msg) {
	case WM_QUIT: // intended fall through
	case WM_DESTROY: PostQuitMessage(EXIT_SUCCESS); break;
	default: return kCoreHandle->MessageHandler(hWnd, msg, wParam, lParam);
	}
	return 0;
}
