#include "dxManager.h"

DXManager::DXManager() :
	_device(nullptr),
	_deviceContext(nullptr),
	_swapChain(nullptr),
	_renderTargetView(nullptr)
{
	_clearColor[0] = 0.6f;
	_clearColor[1] = 0.6f;
	_clearColor[2] = 0.6f;
	_clearColor[3] = 1.f;
}

bool DXManager::Initialize(HWND hwnd, UINT screenWidth, UINT screenHeight)
{
	DXGI_SWAP_CHAIN_DESC swapChainDesc;
	ZeroMemory(&swapChainDesc, sizeof(swapChainDesc));
	swapChainDesc.BufferCount = 1;
	swapChainDesc.BufferDesc.Width = screenWidth;
	swapChainDesc.BufferDesc.Height = screenHeight;
	swapChainDesc.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	swapChainDesc.BufferDesc.RefreshRate.Numerator = 0;
	swapChainDesc.BufferDesc.RefreshRate.Denominator = 1;
	swapChainDesc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
	swapChainDesc.OutputWindow = hwnd;
	swapChainDesc.SampleDesc.Count = 1;
	swapChainDesc.SampleDesc.Quality = 0;
	swapChainDesc.Windowed = true;
	swapChainDesc.BufferDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
	swapChainDesc.BufferDesc.Scaling = DXGI_MODE_SCALING_UNSPECIFIED;
	swapChainDesc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;
	swapChainDesc.Flags = 0;

	D3D_FEATURE_LEVEL featureLevel = D3D_FEATURE_LEVEL_11_0;
	if (FAILED(D3D11CreateDeviceAndSwapChain(nullptr, D3D_DRIVER_TYPE_HARDWARE, nullptr,
		0, &featureLevel, 1, D3D11_SDK_VERSION,
		&swapChainDesc, &_swapChain, &_device, nullptr, &_deviceContext))) {
		return Error("failed to create device and swap chain");
	}

	ID3D11Texture2D* backBufferPtr;
	if (FAILED(_swapChain->GetBuffer(0, __uuidof(ID3D11Texture2D), (LPVOID*)&backBufferPtr))) {
		return Error("failed to receive back buffer pointer");
	}

	if (FAILED(_device->CreateRenderTargetView(backBufferPtr, nullptr, &_renderTargetView))) {
		return Error("failed to create render target view");
	}

	backBufferPtr->Release();
	return true;
}

void DXManager::Shutdown()
{
	SAFE_RELEASE(_renderTargetView);
	SAFE_RELEASE(_swapChain);
	SAFE_RELEASE(_deviceContext);
	SAFE_RELEASE(_device);
}

void DXManager::StartFrameRender()
{
	_deviceContext->ClearRenderTargetView(_renderTargetView, _clearColor);
}

void DXManager::EndFrameRender()
{
	_swapChain->Present(0, 0);
}
